<?php

namespace FiloBlu\Flow\Model\Actions\From;

use Exception;
use FiloBlu\Flow\Helper\CategoryHelper;
use FiloBlu\Flow\Helper\Product as ProductHelper;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\CategoryLinkRepository;

/**
 * Class Removetocategory
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Removetocategory
{
    /**
     * @var array
     */
    protected $categoryLookup = [];

    /**
     * @var array
     */
    protected $excludedCategories = [];

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var ProductHelper
     */
    protected $productHelper;

    /**
     * @var CategoryHelper
     */
    protected $categoryHelper;

    /**
     * @var CategoryLinkRepository
     */
    protected $categoryLinkRepository;

    /**
     * Removetocategory constructor.
     * @param ProductRepositoryInterface $productRepository
     * @param CategoryHelper $categoryHelper
     * @param ProductHelper $productHelper
     * @param CategoryLinkRepository $categoryLinkRepository
     */
    public function __construct(
        ProductRepositoryInterface $productRepository,
        CategoryHelper $categoryHelper,
        ProductHelper $productHelper,
        CategoryLinkRepository $categoryLinkRepository
    ) {
        $this->productRepository = $productRepository;
        $this->productHelper = $productHelper;
        $this->categoryHelper = $categoryHelper;
        $this->categoryLinkRepository = $categoryLinkRepository;
    }

    /**
     * @param string $categoriesString single number (58) single string (parka) path of strings (woman/jacket/parka) path of number (2/5/9) separated by |
     * @param Product $product
     * @param $storeId
     * @throws Exception
     */
    public function process($product, $categoriesString, $storeId)
    {
        if(trim($categoriesString) === "##all##") {
            $categoriesToRemove = $product->getCategoryIds();
            //do not remove from category 1
            $categoriesToRemove = array_diff($categoriesToRemove,["1"]);

            /** TODO category one  */
            $product->setCategoryIds([]);
            $product->save();
            return;
        }
        else {
            $categoriesString = trim(str_replace('|',',',$categoriesString), ',');
            $categories = explode(',', $categoriesString);
            $categoryIds = [];

            $existingCategories = $product->getCategoryIds();
            $categoriesToRemove = array_intersect($categories, $existingCategories);
        }

        if(count($categoriesToRemove)==0){
            return;
        }

        foreach($categoriesToRemove as $cat) {
            $this->categoryLinkRepository->deleteByIds($cat,$product->getSku());
        }
    }



}
